var __awaiter = this && this.__awaiter || function (thisArg, _arguments, P, generator) {
  function adopt(value) {
    return value instanceof P ? value : new P(function (resolve) {
      resolve(value);
    });
  }

  return new (P || (P = Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }

    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }

    function step(result) {
      result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
    }

    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
};

import { withDefaultProps } from '../../utils/with-default-props';
import { useSpring, animated } from '@react-spring/web';
import { useDrag } from 'react-use-gesture';
import { getScrollParent } from '../../utils/get-scroll-parent';
import React, { useRef, useState } from 'react';
import { supportsPassive } from '../../utils/supports-passive';
import { convertPx } from '../../utils/convert-px';
import { rubberbandIfOutOfBounds } from '../../utils/rubberband';
import { sleep } from '../../utils/sleep';
var classPrefix = "adm-pull-to-refresh";
var PullStatus;

(function (PullStatus) {
  PullStatus[PullStatus["idle"] = 0] = "idle";
  PullStatus[PullStatus["thresholdMet"] = 1] = "thresholdMet";
  PullStatus[PullStatus["refreshing"] = 2] = "refreshing";
  PullStatus[PullStatus["complete"] = 3] = "complete";
})(PullStatus || (PullStatus = {}));

export var defaultProps = {
  pullingText: '下拉刷新',
  releaseText: '释放立即刷新',
  refreshingText: '加载中……',
  completeText: '刷新成功',
  completeDelay: 500,
  onRefresh: function onRefresh() {}
};
export var PullToRefresh = withDefaultProps(defaultProps)(function (props) {
  var _a, _b;

  var headHeight = (_a = props.headHeight) !== null && _a !== void 0 ? _a : convertPx(40);
  var threshold = (_b = props.threshold) !== null && _b !== void 0 ? _b : convertPx(60);

  var _useState = useState(PullStatus.idle),
      status = _useState[0],
      setStatus = _useState[1];

  var _useSpring = useSpring(function () {
    return {
      from: {
        height: 0
      },
      config: {
        tension: 300,
        friction: 30,
        clamp: true
      }
    };
  }),
      springStyles = _useSpring[0],
      api = _useSpring[1];

  var elementRef = useRef(null);
  var pullingRef = useRef(false);

  function doRefresh() {
    return __awaiter(this, void 0, void 0, /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
      var _this = this;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              api.start({
                height: headHeight
              });
              setStatus(PullStatus.refreshing);
              setStatus(PullStatus.refreshing);
              _context2.prev = 3;
              _context2.next = 6;
              return props.onRefresh();

            case 6:
              setStatus(PullStatus.complete);
              _context2.next = 13;
              break;

            case 9:
              _context2.prev = 9;
              _context2.t0 = _context2["catch"](3);
              setStatus(PullStatus.idle);
              throw _context2.t0;

            case 13:
              if (!(props.completeDelay > 0)) {
                _context2.next = 16;
                break;
              }

              _context2.next = 16;
              return sleep(props.completeDelay);

            case 16:
              api.start({
                to: function to(next) {
                  return __awaiter(_this, void 0, void 0, /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
                    return regeneratorRuntime.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            _context.next = 2;
                            return next({
                              height: 0
                            });

                          case 2:
                            _context.next = 4;
                            return next({
                              height: 0
                            });

                          case 4:
                            setStatus(PullStatus.idle);

                          case 5:
                          case "end":
                            return _context.stop();
                        }
                      }
                    }, _callee);
                  }));
                }
              });

            case 17:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, null, [[3, 9]]);
    }));
  }

  useDrag(function (state) {
    if (status === PullStatus.refreshing || status === PullStatus.complete) return;
    var event = state.event;

    if (state.last) {
      pullingRef.current = false;

      if (status === PullStatus.thresholdMet) {
        doRefresh();
      } else {
        api.start({
          height: 0
        });
      }

      return;
    }

    var _state$movement = state.movement,
        x = _state$movement[0],
        y = _state$movement[1];

    if (state.first) {
      var element = elementRef.current;
      if (!element) return;
      var scrollParent = getScrollParent(element);
      if (!scrollParent) return;
      var top = 'scrollTop' in scrollParent ? scrollParent.scrollTop : scrollParent.pageYOffset;

      if (top <= 0 && y > 0) {
        pullingRef.current = true;
      }
    }

    if (!pullingRef.current) return;

    if (typeof event.cancelable !== 'boolean' || event.cancelable) {
      event.preventDefault();
    }

    event.stopPropagation();
    var height = Math.max(rubberbandIfOutOfBounds(y, 0, 0, headHeight * 5, 0.5), 0);
    api.start({
      height: height
    });
    setStatus(height > threshold ? PullStatus.thresholdMet : PullStatus.idle);
  }, {
    useTouch: true,
    axis: 'y',
    domTarget: elementRef,
    eventOptions: supportsPassive ? {
      passive: false
    } : false
  });
  return /*#__PURE__*/React.createElement(animated.div, {
    ref: elementRef,
    className: classPrefix
  }, /*#__PURE__*/React.createElement(animated.div, {
    style: springStyles,
    className: classPrefix + "-head"
  }, /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-head-content",
    style: {
      height: headHeight
    }
  }, status === PullStatus.idle && props.pullingText, status === PullStatus.thresholdMet && props.releaseText, status === PullStatus.refreshing && props.refreshingText, status === PullStatus.complete && props.completeText)), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-content"
  }, props.children));
});